
---------------------------------------------------------------------
-- Prepare Demo
---------------------------------------------------------------------

*) open shell and go to local directory where ords.war is located
cd /Users/daust/ords
## java -jar will be in German => change language setting to english
export JAVA_TOOL_OPTIONS="-Duser.language=en"

---------------------------------------------------------------------
-- Administration of REST in SQL Developer
---------------------------------------------------------------------

*) Create a REST user or enable a schema for RESTful access

-- connect as a DBA user

-- create user
CREATE USER ordstest IDENTIFIED BY test123;
GRANT "CONNECT" TO ordstest;
GRANT "RESOURCE" TO ordstest;
GRANT UNLIMITED TABLESPACE TO ordstest;

-- connect as user ordstest

-- REST enable the schema
DECLARE
  PRAGMA AUTONOMOUS_TRANSACTION;
BEGIN

    ORDS.ENABLE_SCHEMA(p_enabled => TRUE,
                       p_schema => 'ORDSTEST',
                       p_url_mapping_type => 'BASE_PATH',
                       p_url_mapping_pattern => 'ordstest',
                       p_auto_rest_auth => FALSE);
    commit;
END;
/

*) Create an administrator user

- create a user in the local ORDS user filestore to enable sql dev to 
- manage the restful definitions
java -jar /Users/daust/ords/ords.war user restadmin "SQL Developer"

- Look at the user which was created in the credentials file
vi /Users/daust/Dropbox/app/ords.3.0/ords/conf/ords/credentials

/*
## nur die Rolle, um ORDS über den SQL Developer zu administrieren
java -jar ords.war user listeneradmin "Listener Administrator"

## Multiple Rollen zuweisen
java -jar ords.war user dietmar "SQL Developer,Listener Administrator"

##!!! Probleme, wenn user für Administration mit "_" eingerichtet wird!
## user ords_admin funktioniert NICHT, 
## user ordsadmin funktioniert!!!

*/

*) Configure SQL Developer

- Activate View > REST Data Services > Development

- Login: 
	- Connection Name: demo
	- Username: restadmin
	- http/https: http
	- hostname: localhost
	- Port: 8080
	- Server Path: /ords
	- Schema/Workspace: ordstest

*) Retrieve configuration from server

*) store backup in filesystem as zip file

----------------------------
-- => RETURN TO PRESENTATION
----------------------------


---------------------------------------------------------------------
-- Use Cases
---------------------------------------------------------------------

*) Install Restful demo services as ordstest in sqlplus

@module-demo-rest-definitions.sql
@module-demo.protected-rest-definitions.sql

*) Reload from Server

*) Use Links for navigation

- Test and check the navigation links (and look at definition)
http://localhost:8080/ords/ordstest/demo/departments/
http://localhost:8080/ords/ordstest/demo/departments/10
http://localhost:8080/ords/ordstest/demo/departments/10/employees

*) create new department using POST

- with Advanced Rest Client (Google Chrome)

POST http://localhost:8080/ords/ordstest/demo/departments/
Content-Type: application/json
{"deptno":99,"dname":"Accounting","loc":"Berlin"}

Alternatively: 
curl -X POST -d deptno=99 -d dname="Accounting" -d loc="Berlin" http://localhost:8080/ords/ordstest/demo/departments/

- Show result (using ARC with GET request)
http://localhost:8080/ords/ordstest/demo/departments/99

*) Modify row using PUT

- Advanced Rest Client (Google Chrome)

PUT http://localhost:8080/ords/ordstest/demo/departments/99
Content-Type: application/json
{"dname":"Accounting","loc":"Cologne"}

Alternatively: 
curl -i -H 'Content-Type: application/json' -X PUT localhost:8080/ords/ordstest/demo/departments/99 -d '{"dname":"Accounting","loc":"Cologne"}'

- Show result: 
http://localhost:8080/ords/ordstest/demo/departments/99
curl -i http://localhost:8080/ords/ordstest/demo/departments/99

*) DELETE row
- mit CURL

curl -i -X DELETE http://localhost:8080/ords/ordstest/demo/departments/99

----------------------------
-- => RETURN TO PRESENTATION
----------------------------

*) Display cgi-environment

in plsql handlers: 

http://localhost:8080/ords/ordstest/demo/cgi-env

*) Use input / output parameters
// Test PUT on demo/employees/{empno}

- Advanced Rest Client (Google Chrome)

PUT http://localhost:8080/ords/ordstest/demo/employees/7839
Content-Type: application/json
{"ename":"KING","job":"PRESIDENT","mgr": null}

Alternatively: 
curl -i -H 'Content-Type: application/json' -X PUT http://localhost:8080/ords/ordstest/demo/employees/7839 -d '{"ename":"KING","job":"PRESIDENT","mgr": null}'

- show logging information
select text, scope, user_name,extra 
  from LOGGER_LOGS_5_MIN
 order by id desc;


----------------------------
-- => RETURN TO PRESENTATION
----------------------------

---------------------------------------------------------------------
-- Authentication and Authorization
---------------------------------------------------------------------

*) Remove role "HR Administrator" and privilege to protect the module

sqlplus ordstest@vm1:1521/xe

begin
  ORDS.delete_privilege( p_name => 'demo.protected');
  ORDS.delete_role( p_role_name => 'HR Administrator');
  commit;
end;
/

- test unprotected url
http://localhost:8080/ords/ordstest/demo-protected/departments/

- protect url using a script

declare
  l_roles    owa.vc_arr;
  l_patterns owa.vc_arr;
  l_modules  owa.vc_arr;
begin
  ----------------------------------------------------------------------------
  -- cleanup
  ----------------------------------------------------------------------------
  ORDS.delete_privilege( p_name => 'demo.protected');
  ORDS.delete_role( p_role_name => 'HR Administrator');

  ----------------------------------------------------------------------------
  -- create role
  ----------------------------------------------------------------------------
  ORDS.CREATE_ROLE( p_role_name => 'HR Administrator');
  
  ----------------------------------------------------------------------------
  -- protect module
  ----------------------------------------------------------------------------
  l_roles(1) := 'HR Administrator';

  -- Liste der URL-Patterns, die geschützt werden sollen.
  --l_patterns(1) := '/emp/list/*';
  l_modules(1) := 'demo.protected';
  ords.define_privilege(
    p_privilege_name => 'demo.protected'
   ,p_roles =>          l_roles
   ,p_patterns =>       l_patterns     
   ,p_modules =>        l_modules      
   ,p_description =>    'only users with role HR Administrator are allowed'
   ,p_label =>          'Demo Privilege'
  );
  
  commit;
end;
/

- check definition with GUI

- create user hradmin having the role: "HR Administrator"
java -jar /Users/daust/ords/ords.war user hradmin "HR Administrator"

- reload current page and login:
http://localhost:8080/ords/ordstest/demo-protected/departments/

- from glassfish
Manfred.Mustermann

-- can re-login using a different user:
--http://localhost:8080/ords/ordstest/sign-in/?r=demo-protected%2Fdepartments%2F

- logout user: 
http://localhost:8080/ords/ordstest/signed-out/

- reload the page
http://localhost:8080/ords/ordstest/demo-protected/departments/

----------------------------
-- => RETURN TO PRESENTATION
----------------------------

---------------------------------------------------------------------
-- AUTO REST
---------------------------------------------------------------------

*) Auto-REST enable a table

DECLARE
  PRAGMA AUTONOMOUS_TRANSACTION;
BEGIN

    ORDS.ENABLE_OBJECT(p_enabled => TRUE,
                       p_schema => 'ORDSTEST',
                       p_object => 'DEPT',
                       p_object_type => 'TABLE',
                       p_object_alias => 'dept',
                       p_auto_rest_auth => FALSE);
    
    commit;
END;
/


- check which ones are enabled?

-- show publicly visible views
SELECT *
  FROM all_objects
 WHERE owner = 'ORDS_METADATA' AND object_type = 'VIEW'
    and object_name not like 'ALL%';
 
-- modules
select * 
  from user_ords_modules;

-- handlers
select * 
  from user_ords_handlers;

-- show all service ... INCLUDING the autorest objects
select *
  from USER_ORDS_SERVICES
 where base_path='/dept/';
 
*) create a new department
curl -i -X POST -d deptno=99 -d dname="Accounting" -d loc="Berlin" http://localhost:8080/ords/ordstest/dept/

- look at the new department
curl http://localhost:8080/ords/ordstest/dept/99

- Modify row using PUT
curl -i -H 'Content-Type: application/json' -X PUT localhost:8080/ords/ordstest/dept/99 -d '{"dname":"Accounting","loc":"Cologne"}'

- show result
curl http://localhost:8080/ords/ordstest/dept/99

*) DELETE Datensatz
curl -i -X DELETE http://localhost:8080/ords/ordstest/dept/99

 

